#include <L298NX2.h>

#define ECHO_LEFT 10
#define ECHO_FRONT 11
#define ECHO_RIGHT 12

#define TRIGGER 2

#define POTENTIOMETER A4

#define MIN_SPEED 80
#define MAX_SPEED 255
#define MAX_POTEN 1023
#define TURN_SPEED 80
#define TURN_TIME 380
#define STOP_TIME 500
#define TURN_DIST 25

#define EN_A 3
#define IN1_A 5
#define IN2_A 6
#define IN1_B 7
#define IN2_B 8
#define EN_B 9

L298NX2 motors(EN_A, IN1_A, IN2_A, EN_B, IN1_B, IN2_B);

int d_left, d_front, d_right;
float poten = 0;

void setup() {
  pinMode(TRIGGER, OUTPUT);
  pinMode(ECHO_LEFT, INPUT);
  pinMode(ECHO_FRONT, INPUT);
  pinMode(ECHO_RIGHT, INPUT);
  pinMode(POTENTIOMETER, INPUT);

  poten = analogRead(POTENTIOMETER);
  int start_vit = (float)(poten / MAX_POTEN) * (MAX_SPEED - MIN_SPEED) + MIN_SPEED;
  motors.setSpeed(start_vit);
  motors.forward();
}

long getDist(int trigger, int echo) {
  digitalWrite(trigger, LOW);
  delay(5);
  digitalWrite(trigger, HIGH);
  delay(10);
  digitalWrite(trigger, LOW);

  long duration = pulseIn(echo, HIGH, 12000);
  return (duration / 2) * 0.03432;
}

void turnRight() {
  motors.stop();
  motors.setSpeed(TURN_SPEED);
  motors.forwardA();
  motors.backwardB();
  delay(TURN_TIME);
  motors.stop();
  delay(STOP_TIME);

  float new_poten = analogRead(POTENTIOMETER);
  changeSpeed(new_poten);
}

void turnLeft() {
  motors.stop();
  motors.setSpeed(TURN_SPEED);
  motors.backwardA();
  motors.forwardB();
  delay(TURN_TIME);
  motors.stop();
  delay(STOP_TIME);

  float new_poten = analogRead(POTENTIOMETER);
  changeSpeed(new_poten);
}

void changeSpeed(float new_poten) {
  poten = new_poten;
  int new_vit = (float)(poten / MAX_POTEN) * (MAX_SPEED - MIN_SPEED) + MIN_SPEED;
  motors.setSpeed(new_vit);
  motors.forward();
}

void loop() {
  float new_poten = analogRead(POTENTIOMETER);

  if (abs(new_poten - poten) >= 5) {
    changeSpeed(new_poten);
  }

  d_front = getDist(TRIGGER, ECHO_FRONT);

  if (d_front < TURN_DIST && d_front > 0) {
    d_left = getDist(TRIGGER, ECHO_LEFT);
    d_right = getDist(TRIGGER, ECHO_RIGHT);

    if (d_left == 0) {
      turnLeft();
    } else if (d_right == 0) {
      turnRight();
    } else if (d_left > d_right) {
      turnLeft();
    } else {
      turnRight(); 
    }
  }
}
